"""
-------------------------------------------------------------------------------
Name:    Fishbone Connection Lines
Purpose: This script is intended to be used as an example for building line
    geometry to create a fishbone analysis
Author:      John Ehlen
Created:     06/01/2020
Copyright:   (c) John J. Ehlen 2020

This script assumes other steps not needing python have been completed. The
intention is to take a set of original address points and a set of geocoded
address points and build line geometry between them. There are many ways to
achieve this, including using SHAPE@ tokens, however I chose to use attributes
in the original address feature class that propogated through the geocoding process.

Further analysis can also be achieved depending on data quality and needs. It is possible
to autmate the entire process from Dual Range U.S. Address locator creation to
the creation of relationship classes between the geocoded address
points, the connection lines, the original address points, and the road centerlines.
For help with any of these topics reach out to me at John@johnehlen.com
-------------------------------------------------------------------------------
"""

# Truncate old connection lines feature class records
arcpy.management.TruncateTable('Connection_Lines')

# Create empty list to house connection lines
connection_line_list = []

# Open search cursor to retrieve information from each row
with arcpy.da.SearchCursor('GeocodedPoint',["Match_addr", "POINT_X", "POINT_Y", "DisplayX", "DisplayY"]) as in_table:

  # Iterate through each row in the table
  for row in in_table:

    # we will assign the values of "POINT_X" and "POINT_Y" to the variables "xStart" and "yStart"
    xStart = row[1]
    yStart = row[2]

    # we will assign the values of "DisplayX and "DiisplayY" to the variables "xEnd" and "yEnd"
    xEnd = row[3]
    yEnd = row[4]

    # Next we use the X,Y coordinates to build starting and endings points. These will be the vertices for our polyline.
    startPoint = arcpy.Point(xStart,yStart)
    endPoint = arcpy.Point(xEnd,yEnd)

    # Next we create an array to house the points for the polyline
    vertices = arcpy.Array([startPoint,endPoint])

    # Create polyline object from point array
    line = arcpy.Polyline(vertices)

    #NOTE - The above steps inside the for loop can be condensed into the following line:
        #line = arcpy.Polyline(arcpy.Array([arcpy.Point(row[1], row[2]), arcpy.Point(row[3], row[4])]))

    # Add polyline geometry object to list. The insert cursor is expecting a list
    # and this will allow us to pass values easy
    feature = [line]

    # Use list extend method to add row value "Match_addr" from cursor to feature
    feature.extend([row[0]])

    # Append new feature to the connection line list
    connection_line_list.append(feature)



# Use insert cursor to insert new polyline features stored in connection_line_list
with arcpy.da.InsertCursor('Connection_Lines',["SHAPE@", "Match_addr"]) as insertCursor:
  for line in connection_line_list:
    insertCursor.insertRow(line)